#!/bin/bash
# Example application for AppOptics on Linux platform
# Usage:
#   ./run.sh <your-service-key>:example-clr-linux

cd $(dirname "$0")
PROGNAME=$(basename $0)
ROOT_DIR=`pwd`

# Write to a log file
PROGNAME_LOG_FILE=${PROGNAME}.log
exec >  >(tee -ia ${PROGNAME_LOG_FILE})
exec 2> >(tee -ia ${PROGNAME_LOG_FILE} >&2)

# Pass service key as parameter to the script or use service key already set in environment
export APPOPTICS_SERVICE_KEY=${APPOPTICS_SERVICE_KEY:-${1}}

# Configuration to build
CONFIGURATION=Release

# Publish directory for the application
PUBLISH_DIR=./bin/${CONFIGURATION}/netcoreapp3.1/publish

# agent.config configuration file
CONFIG_FILE=${PUBLISH_DIR}/agent.config

# Log files location
LOG_FILES_DIR=/var/tmp/appoptics

PID_FILE=example.program.pid

usage()
{
  cat << EOU

  Usage:  ${0} <your-service-key>:example-clr-linux

EOU
}

display_error_no_service_key()
{
    echo "ERROR: Please set the APPOPTICS_SERVICE_KEY before you run the example application"
    usage
    exit 1
}

check_requirements()
{
    # Verify that APPOPTICS_SERVICE_KEY enviroment variable was provided
    case ${APPOPTICS_SERVICE_KEY} in
       (*[![:blank:]]*) echo "APPOPTICS_SERVICE_KEY environment variable is set";;
       (*) display_error_no_service_key
    esac
}

stop_application()
{
    if [ -s "${PID_FILE}" ]; then
        pkill -9 --pidfile "${PID_FILE}"
        rm ${PID_FILE}
    fi
}

clean_build()
{
    echo Cleaning build artifacts ...
    rm -rf ./bin ./obj
    dotnet clean --configuration ${CONFIGURATION}
    echo cleaned.
}

build_application()
{
    PKG_VERSION=$(ls /package_sources/*.nupkg | sed 's/.*AppOptics.Instrumentation.\(.*\).nupkg/\1/g')
    sed -i 's/\(Include="AppOptics.Instrumentation" Version="\).*\(" \/>\)/\1'$PKG_VERSION'\2/g' coretestapp.csproj
    
    stop_application
    clean_build

    echo Building application ...
    dotnet publish --configuration ${CONFIGURATION}
    echo built.
}

update_service_key()
{
   sed -i "s/\(key=\"ServiceKey\"\)*\(value=\"\"\)/value=\"${APPOPTICS_SERVICE_KEY}\"/" ${CONFIG_FILE}
}

run_application()
{
    echo Change directory to publish directory ${PUBLISH_DIR}
    pushd ${PUBLISH_DIR}

    echo Launching application in background ...
    dotnet coretestapp.dll &
    echo $! > ${PID_FILE}
    echo launched.

    echo Waiting until application is fully initialized ...
    sleep 10
    echo done.

    echo Sending HTTP requests to application ...
    for i in {1..5}; do
        echo --------------------- Sending HTTP request to application ...
        echo
        curl -H "Accept: application/xml" -H "Content-Type: application/xml" -X GET http://localhost:5000
        echo
        echo --------------------- HTTP request sent.
    done
    echo sent.

    echo Congratulations! You have run the example application.
    echo Application log files can be found in ${LOG_FILES_DIR} directory.
    show_log_files
}

show_log_files()
{
    for LOG in $(ls ${LOG_FILES_DIR}/DotNetAgentLog*.log); do
        echo "Content of $LOG"
        echo "============================="
        cat $LOG
        echo "============================="
        echo
    done
}

main()
{
    check_requirements
    stop_application
    build_application
    update_service_key
    run_application
    stop_application
}

main

# clean up
echo "Cleaning up $ROOT_DIR"
cd $ROOT_DIR
rm -rf ./bin ./obj
